/*
 * Windows CE program to encrypt/decrypt a file using RC4 in stream
 * mode, given a key entered by the user. The UI is very primitive.
 *  (c) 2008 Warren Toomey, GPL3 license.
 */

#include "wktwallet.h"
#include <commctrl.h>
#include <commdlg.h>
#include <projects.h>
#include "rc4.h"

static HINSTANCE hInst = NULL;	// Local copy of hInstance.
static HWND hwndMain = NULL;	// Handle to main window
static HWND hwndCB = NULL;	// Handle to a commandbar.

static WCHAR wszAppName[] = L"wkt Wallet";	// App 's full name
static WCHAR wszTitle[] = L"wktWallet"; 	// App 's title
static WCHAR version[] = L"1.0";

#define KEYSIZE 24
static WCHAR key[KEYSIZE]; 	// Key entered in by user


/*
 * Deal with the dialog box to get the key from the user
 */
static LRESULT CALLBACK MainDlgProc(HWND hwndDlg, UINT uMsg,
				        WPARAM wParam, LPARAM lParam)
{
  switch (uMsg) {
    case WM_INITDIALOG:		// Ensure we get keyboard focus
      return (TRUE);

    case WM_COMMAND:
      switch (GET_WM_COMMAND_ID(wParam, lParam)) {

        case IDOK:				// Get the key from the user
          memset(key, 0, sizeof(key));
          GetDlgItemText(hwndDlg, IDG_KEYVAL, key, KEYSIZE);

        case IDCANCEL:				// and close down the dialog box
	  EndDialog(hwndDlg, TRUE);
          return TRUE;
      }
      break;

    case WM_CLOSE:				// Close us down abnormally
      EndDialog(hwndDlg, 0);
      return TRUE;

    case WM_DESTROY:
      break;
  }
  return FALSE;
}

/*
 * Encrypt or decrypt a file, saving the result in outFile.
 */
void endeCrypt(WCHAR * inFile, WCHAR *outFile)
{
#define BUFSIZE 256
  HANDLE inHandle, outHandle;
  DWORD dwBytesRead, dwBytesWritten;
  struct rc4_state rc4State;
  unsigned char inbuf[BUFSIZE];
  unsigned char outbuf[BUFSIZE];
  unsigned char realkey[KEYSIZE], *r;
  int i;

  // Open the existing file.
  inHandle = CreateFile(inFile,
			  GENERIC_READ, // Open for reading
			  0, 		// Do not share
			  NULL, 	// No security
			  OPEN_EXISTING, // Existing file only
			  FILE_ATTRIBUTE_NORMAL, // Normal file
			  NULL);

  if (inHandle == INVALID_HANDLE_VALUE) {
    ShowInfo(hwndMain, L"Err: unable to read file");
    return;
  }

  // Create a new file, overwriting any existing one.
  outHandle = CreateFile(outFile,
			   GENERIC_WRITE, // Open for writing
			   0, 		  // Do not share
			   NULL, 	  // No security
			   OPEN_ALWAYS,   // Open or create
			   FILE_ATTRIBUTE_NORMAL, // Normal file
			   NULL);

  if (outHandle == INVALID_HANDLE_VALUE) {
    ShowInfo(hwndMain, L"Err: unable to create new file");
    return;
  }

  // Initialise the key. Because the key is a WCHAR, half of the
  // bytes are probably 0. We copy only the non-zero octets into
  // the real key.
  memset(realkey, 0, sizeof(realkey));
  for (i = 0, r = realkey; i < KEYSIZE; i++)
    if (key[i] != 0)
      *r++ = (u_char) key[i] & 0xff;

  // Use the realkey to initialise RC4
  rc4_init(&rc4State, realkey, i);

  // Read and encrypt data from inHandle to outHandle
  while (ReadFile(inHandle, inbuf, BUFSIZE, &dwBytesRead, NULL)) {
    if (dwBytesRead < 1) break;
    rc4_crypt(&rc4State, inbuf, outbuf, dwBytesRead);
    WriteFile(outHandle, outbuf, dwBytesRead, &dwBytesWritten, NULL);
  };

  // Close both files.
  CloseHandle(inHandle);
  CloseHandle(outHandle);

  return;
}

/*
 * Process the Menu actions
 */
LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  WCHAR wszFile[MAX_PATH] = L"\0";
  OPENFILENAME ofn = {0};
  WCHAR outFile[MAX_PATH];
  WCHAR *fullstop, *underscore;

  ofn.lStructSize = sizeof(ofn);
  ofn.hwndOwner = hwnd;
  ofn.lpstrFile = wszFile;
  ofn.nMaxFile = MAX_PATH;

  switch (msg) {
  case WM_CREATE:
    hwndCB = CommandBar_Create(hInst, hwnd, 1);
    CommandBar_InsertMenubar(hwndCB, hInst, IDR_MAIN_MENU, 0);
    break;

  case WM_COMMAND:
    switch (GET_WM_COMMAND_ID(wParam, lParam)) {
    case IDM_ENCRYPT:
	// Prompt the user and get the input filename
	ofn.lpstrFilter = L"All (*.*)\0*.*\0";
	ofn.lpstrTitle = L"Encrypt File";
	ofn.Flags = OFN_EXPLORER;
	if (!GetOpenFileName(&ofn)) break;

	// Find the last period character
	fullstop = wcsrchr(ofn.lpstrFile, L'.');
	if (fullstop == NULL) break;

	// Set it to be an underscore
	*fullstop = L'_';

	// Create output filename with .enc appended
	wsprintf(outFile, L"%s.enc", ofn.lpstrFile);

	// Set the ofn.lpstrFile back to its original value
	*fullstop = L'.';

	// Get the key if we don 't have one. Give up if none arrives.
	if (key[0] == 0)
	    DialogBox(hInst, MAKEINTRESOURCE(IDG_SETKEY), NULL,
		      (DLGPROC) MainDlgProc);
	if (key[0] == 0) break;

	// Encrypt the file
	endeCrypt(ofn.lpstrFile, outFile);
        UpdateWindow(hwndMain);
	break;

    case IDM_DECRYPT:
	// Prompt the user and get the input filename
	ofn.lpstrFilter = L"All (*.enc)\0*.enc\0";
	ofn.lpstrTitle = L"Decrypt File";
	ofn.Flags = OFN_EXPLORER;
	ofn.lpstrDefExt = L"txt";
	if (!GetSaveFileName(&ofn)) break;

	// Copy the input filename to the output
	wcsncpy(outFile, ofn.lpstrFile, MAX_PATH - 1);

	// Find the last period character and underscore
	fullstop = wcsrchr(outFile, L'.');
	if (fullstop == NULL) break;
	underscore = wcsrchr(outFile, L'_');
	if (underscore == NULL) break;

	// Truncate the file at fullstop, convert _ into .
	*fullstop = L'\0';
	*underscore = L'.';

	// Get the key if we don 't have one. Give up if none arrives.
	if (key[0] == 0)
	    DialogBox(hInst, MAKEINTRESOURCE(IDG_SETKEY), NULL,
		      (DLGPROC) MainDlgProc);
	if (key[0] == 0) break;

	// Decrypt the file
	endeCrypt(ofn.lpstrFile, outFile);
        UpdateWindow(hwndMain);
	break;

    case IDM_SETKEY:		// Show the set key dialog box
	DialogBox(hInst, MAKEINTRESOURCE(IDG_SETKEY), NULL,
		  (DLGPROC) MainDlgProc);
	break;

    case IDM_ABOUT:
	wsprintf(outFile, L"%s, Version %s", wszAppName, version);
        ShowInfo(hwnd, outFile);
	break;

    case IDM_QUIT:
	CommandBar_Destroy(hwndCB);
	DestroyWindow(hwnd);
	return (0);

    case IDOK:
      SendMessage(hwnd, WM_CLOSE, 0, 0);
      break;

    default:
      return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    break;

  case WM_CLOSE:
    CommandBar_Destroy(hwndCB);
    DestroyWindow(hwnd);
    break;

  case WM_DESTROY:
    PostQuitMessage(0);
    break;

  default:
    return DefWindowProc(hwnd, msg, wParam, lParam);
  }
  return 0;
}


/*
 * Initialise the instance.
 */

BOOL InitInstance(HINSTANCE hInstance, int CmdShow)
{
  hInst = hInstance;

  // Create the application 's window
  hwndMain = CreateWindow(wszAppName, wszTitle,
			    WS_VISIBLE | WS_CAPTION | WS_SYSMENU,
			    CW_USEDEFAULT, CW_USEDEFAULT, 300, 150,
			    NULL, NULL, hInstance, NULL);
  if (!hwndMain) return FALSE;

  // and show it
  ShowWindow(hwndMain, CmdShow);
  UpdateWindow(hwndMain);
  return TRUE;
}

/*
 * Initialise the application
 */

ATOM InitApplication(HINSTANCE hInstance)
{
  WNDCLASS wc;

  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc = (WNDPROC) WndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hIcon = NULL;
  wc.hInstance = hInstance;
  wc.hCursor = NULL;
  wc.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
  wc.lpszMenuName = NULL;
  wc.lpszClassName = wszAppName;
  return RegisterClass(&wc);
}

/*
 * Main Program
 */
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
		       LPWSTR lpCmdLine, int CmdShow)
{
  MSG msg;
  HWND hPrjWnd = NULL;
  HACCEL hAccel = NULL;

  // Check if program is running.If it 's running then focus on the window.
  hPrjWnd = FindWindow(wszAppName, wszTitle);
  if (hPrjWnd) {
    SetForegroundWindow(hPrjWnd); return 0;
  }

  if (!hPrevInstance) {
    if (!InitApplication(hInstance)) return FALSE;
  }

  if (!InitInstance(hInstance, CmdShow)) return FALSE;

  hAccel = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDR_ACCELERATOR));

  while (GetMessage(&msg, NULL, 0, 0)) {
    if (!TranslateAccelerator(hwndMain, hAccel, &msg)) {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
  }

  return msg.wParam;
}
